@@ -24,7 +24,7 @@
 #include "DasherButtons.h"
 #include "DynamicFilter.h"
 #include "EyetrackerFilter.h"
-#include "OneButtonFilter.h"
+//#include "OneButtonFilter.h"
 #include "OneDimensionalFilter.h"
 #include "StylusFilter.h"
 #include "TwoButtonDynamicFilter.h"
@@ -70,6 +70,8 @@ CDasherInterfaceBase::CDasherInterfaceBase()
   m_bGlobalLock = false;
   m_bShutdownLock = false;
 
+  m_bRedrawScheduled = false;
+
   m_pEventHandler = new CEventHandler(this);
  
   strCurrentContext = ". ";
@@ -148,7 +150,7 @@ CDasherInterfaceBase::~CDasherInterfaceBase() {
   delete m_pEventHandler;
 }
 
-void CDasherInterfaceBase::PreSetNotify(int iParameter) {
+void CDasherInterfaceBase::PreSetNotify(int iParameter, const std::string &sNewValue) {
 
   // FIXME - make this a more general 'pre-set' event in the message
   // infrastructure
@@ -156,12 +158,21 @@ void CDasherInterfaceBase::PreSetNotify(int iParameter) {
   switch(iParameter) {
   case SP_ALPHABET_ID: 
     // Cycle the alphabet history
-    
-    SetStringParameter(SP_ALPHABET_4, GetStringParameter(SP_ALPHABET_3));
-    SetStringParameter(SP_ALPHABET_3, GetStringParameter(SP_ALPHABET_2));
-    SetStringParameter(SP_ALPHABET_2, GetStringParameter(SP_ALPHABET_1));
-    SetStringParameter(SP_ALPHABET_1, GetStringParameter(SP_ALPHABET_ID));
-    
+    if(GetStringParameter(SP_ALPHABET_ID) != sNewValue) {
+      if(GetStringParameter(SP_ALPHABET_1) != sNewValue) {
+	if(GetStringParameter(SP_ALPHABET_2) != sNewValue) {
+	  if(GetStringParameter(SP_ALPHABET_3) != sNewValue)
+	    SetStringParameter(SP_ALPHABET_4, GetStringParameter(SP_ALPHABET_3));
+	  
+	  SetStringParameter(SP_ALPHABET_3, GetStringParameter(SP_ALPHABET_2));
+	}
+	
+	SetStringParameter(SP_ALPHABET_2, GetStringParameter(SP_ALPHABET_1));
+      }
+      
+      SetStringParameter(SP_ALPHABET_1, GetStringParameter(SP_ALPHABET_ID));
+    }
+
     break;
   }
 }
@@ -218,52 +229,12 @@ void CDasherInterfaceBase::InterfaceEventHandler(Dasher::CEvent *pEvent) {
     case SP_INPUT_DEVICE:
       CreateInput();
       break;
-  //   case BP_NUMBER_DIMENSIONS:
-//     case BP_EYETRACKER_MODE:
-//     case BP_KEY_CONTROL:
-//     case BP_BUTTONONESTATIC:
-//     case BP_BUTTONONEDYNAMIC:
-//     case BP_BUTTONMENU:
-//     case BP_BUTTONDIRECT:
-//     case BP_BUTTONFOURDIRECT:
-//     case BP_BUTTONALTERNATINGDIRECT:
-//     case BP_COMPASSMODE:
-//     case BP_CLICK_MODE:
-//       // Delibarate fallthrough.  
-//       // FIXME - Horrible mess below - should really use
-//       // SP_INPUT_FILTER directly for the sake of sanity
-//       if(GetBoolParameter(BP_NUMBER_DIMENSIONS))
-// 	SetLongParameter(LP_INPUT_FILTER, 4); // 1D mode
-//       else if(GetBoolParameter(BP_EYETRACKER_MODE))
-// 	SetLongParameter(LP_INPUT_FILTER, 5);//"Eyetracker Mode"
-//       else if(GetBoolParameter(BP_CLICK_MODE))
-// 	SetLongParameter(LP_INPUT_FILTER, 7);//"Click Mode"
-//       else if(GetBoolParameter(BP_KEY_CONTROL)) {
-// 	// Various button modes
-// 	if(GetBoolParameter(BP_BUTTONONESTATIC))
-// 	  SetLongParameter(LP_INPUT_FILTER, 8);//"One Button Static"
-// 	else if(GetBoolParameter(BP_BUTTONONEDYNAMIC))
-// 	  SetLongParameter(LP_INPUT_FILTER, 6);//"One Button Dynamic"
-// 	else if(GetBoolParameter(BP_BUTTONMENU))
-// 	  SetLongParameter(LP_INPUT_FILTER, 8);//"Button Menu"
-// 	else if(GetBoolParameter(BP_BUTTONDIRECT))
-// 	  SetLongParameter(LP_INPUT_FILTER, 10); //"Three Button Direct"
-// 	else if(GetBoolParameter(BP_BUTTONFOURDIRECT))
-// 	  SetLongParameter(LP_INPUT_FILTER, 11);//"Four Button Direct"
-// 	else if(GetBoolParameter(BP_BUTTONALTERNATINGDIRECT))
-// 	  SetLongParameter(LP_INPUT_FILTER, 12);//"Alternating Direct"
-// 	else if(GetBoolParameter(BP_COMPASSMODE))
-// 	  SetLongParameter(LP_INPUT_FILTER, 13);//"Compass Mode"
-//       }
-//       else
-// 	SetLongParameter(LP_INPUT_FILTER, 3);//"Default"
-//       break;
     case SP_INPUT_FILTER:
       CreateInputFilter();
       break;
     default:
-      break;}
-      
+      break;
+    }
   }
   else if(pEvent->m_iEventType == 2) {
     CEditEvent *pEditEvent(static_cast < CEditEvent * >(pEvent));
@@ -311,10 +282,6 @@ void CDasherInterfaceBase::WriteTrainFilePartial() {
   strTrainfileBuffer = strTrainfileBuffer.substr(100);
 }
 
-// void CDasherInterfaceBase::RequestFullRedraw() {
-//   SetBoolParameter( BP_REDRAW, true );
-// }
-
 void CDasherInterfaceBase::CreateDasherModel() 
 {
   if(!m_AlphIO)
@@ -338,7 +305,13 @@ void CDasherInterfaceBase::CreateDasherModel()
       m_pDasherModel = 0;
     }
 
-    m_pDasherModel = new CDasherModel(m_pEventHandler, m_pSettingsStore, this, m_AlphIO);
+    if(m_deGameModeStrings.size() == 0)
+      m_pDasherModel = new CDasherModel(m_pEventHandler, m_pSettingsStore, this, m_AlphIO);
+    else {
+      m_pDasherModel = new CDasherModel(m_pEventHandler, m_pSettingsStore, this, m_AlphIO, true, m_deGameModeStrings[0]);
+      //      m_deGameModeStrings.pop_front();
+    }
+
     m_Alphabet = m_pDasherModel->GetAlphabetNew();
     
     string T = m_Alphabet->GetTrainingFile();
@@ -425,8 +398,6 @@ void CDasherInterfaceBase::Unpause(unsigned long Time) {
 
 void CDasherInterfaceBase::CreateInput() {
 
-  std::cout << "Creating input device: " << GetStringParameter(SP_INPUT_DEVICE) << std::endl;
-
   // FIXME - this shouldn't be the model used here - we should just change a parameter and work from the appropriate listener
 
   if(m_pInput) {
@@ -460,8 +431,7 @@ void CDasherInterfaceBase::NewFrame(unsigned long iTime) {
 	int iNumDeleted = 0;
 	
 	if(m_pInputFilter) {
-	  bChanged = true; // TODO: Actually make CInputFilter::Timer return bool
-	  m_pInputFilter->Timer(iTime, m_pDasherView, m_pDasherModel); // FIXME - need logging stuff here
+	  bChanged = m_pInputFilter->Timer(iTime, m_pDasherView, m_pDasherModel); // FIXME - need logging stuff here
 	}
 	
 	if (iNumDeleted > 0)
@@ -472,8 +442,7 @@ void CDasherInterfaceBase::NewFrame(unsigned long iTime) {
       }
       else {
 	if(m_pInputFilter) {
-	  bChanged = true;
-	  m_pInputFilter->Timer(iTime, m_pDasherView, m_pDasherModel);
+	  bChanged = m_pInputFilter->Timer(iTime, m_pDasherView, m_pDasherModel);
 	}
       }
       
@@ -481,8 +450,11 @@ void CDasherInterfaceBase::NewFrame(unsigned long iTime) {
     }
   }
 
-  Redraw(bChanged);
+  Redraw(bChanged || m_bRedrawScheduled);
+  m_bRedrawScheduled = false;
 
+  // This just passes the time through to the framerate tracker, so we
+  // know how often new frames are being drawn.
   if(m_pDasherModel != 0)
     m_pDasherModel->NewFrame(iTime);
 }
@@ -497,7 +469,7 @@ void CDasherInterfaceBase::Redraw(bool bRedrawNodes) {
   
   if(bRedrawNodes) {
     m_pDasherView->Screen()->SendMarker(0);
-    m_pDasherModel->RenderToView(m_pDasherView,true);
+    m_pDasherModel->RenderToView(m_pDasherView, true);
   }
   
   m_pDasherView->Screen()->SendMarker(1);
@@ -505,8 +477,7 @@ void CDasherInterfaceBase::Redraw(bool bRedrawNodes) {
   bool bDecorationsChanged(false);
   
   if(m_pInputFilter) {
-    bDecorationsChanged = true;
-    m_pInputFilter->DecorateView(m_pDasherView);
+    bDecorationsChanged = m_pInputFilter->DecorateView(m_pDasherView);
   }
   
   if(bRedrawNodes || bDecorationsChanged)
@@ -515,11 +486,15 @@ void CDasherInterfaceBase::Redraw(bool bRedrawNodes) {
 
 void CDasherInterfaceBase::ChangeAlphabet() {
 
+  if(GetStringParameter(SP_ALPHABET_ID) == "") {
+    SetStringParameter(SP_ALPHABET_ID, m_AlphIO->GetDefault());
+    // This will result in ChangeAlphabet() being called again, so
+    // exit from the first recursion
+    return;
+  }
+  
   // Send a lock event
 
-
-  std::cout << "Alphabet set to: ***" << GetStringParameter(SP_ALPHABET_ID) << "***" << std::endl;
-
   WriteTrainFileFull();
 
   // Lock Dasher to prevent changes from happening while we're training.
@@ -627,6 +602,26 @@ void CDasherInterfaceBase::GetColours(std::vector <std::string >*ColourList) {
   m_ColourIO->GetColours(ColourList);
 }
 
+int CDasherInterfaceBase::TrainFileWrapper(string Filename, int iTotalBytes, int iOffset) {
+   // Train the new language model
+    CLockEvent *pEvent;
+    
+    pEvent = new CLockEvent("Training Dasher", true, 0);
+    m_pEventHandler->InsertEvent(pEvent);
+    delete pEvent;
+
+  int iRetVal = TrainFile(Filename, iTotalBytes, iOffset);
+   // Train the new language model
+    
+    
+    pEvent = new CLockEvent("Training Dasher", false, 0);
+    m_pEventHandler->InsertEvent(pEvent);
+    delete pEvent;
+
+
+    return iRetVal;
+}
+
 /*
 	I've used C style I/O because I found that C++ style I/O bloated
 	the Win32 code enormously. The overhead of loading the buffer into
@@ -722,13 +717,12 @@ void CDasherInterfaceBase::ResetNats() {
 
 
 void CDasherInterfaceBase::InvalidateContext(bool bForceStart) {
-
   m_pDasherModel->m_strContextBuffer = "";
 
   Dasher::CEditContextEvent oEvent(10);
   m_pEventHandler->InsertEvent(&oEvent);
 
-  std::string strNewContext(m_pDasherModel->m_strContextBuffer);
+   std::string strNewContext(m_pDasherModel->m_strContextBuffer);
 
   // We keep track of an internal context and compare that to what
   // we are given - don't restart Dasher if nothing has changed.
@@ -744,7 +738,7 @@ void CDasherInterfaceBase::InvalidateContext(bool bForceStart) {
    if(bForceStart || (strNewContext.substr( std::max(static_cast<int>(strNewContext.size()) - iContextLength, 0)) != strCurrentContext.substr( std::max(static_cast<int>(strCurrentContext.size()) - iContextLength, 0)))) {
 
      if(m_pDasherModel != NULL) {
-       if(m_pDasherModel->m_bContextSensitive) {
+       if(m_pDasherModel->m_bContextSensitive || bForceStart) {
  	m_pDasherModel->SetContext(strNewContext);
  	PauseAt(0,0);
        }
@@ -754,10 +748,15 @@ void CDasherInterfaceBase::InvalidateContext(bool bForceStart) {
      WriteTrainFileFull();
    }
 
+  
+
    if(m_pDasherView)
      while( m_pDasherModel->CheckForNewRoot(m_pDasherView) ) {
        // Do nothing
      }
+
+   //   Redraw(true);
+   ScheduleRedraw();
 }
 
 
@@ -780,20 +779,15 @@ void CDasherInterfaceBase::DisconnectNode(int iChild, int iParent) {
 }
 
 void CDasherInterfaceBase::SetBoolParameter(int iParameter, bool bValue) {
-  PreSetNotify(iParameter);
-  
   m_pSettingsStore->SetBoolParameter(iParameter, bValue);
 };
 
 void CDasherInterfaceBase::SetLongParameter(int iParameter, long lValue) { 
-  PreSetNotify(iParameter);
-  
   m_pSettingsStore->SetLongParameter(iParameter, lValue);
 };
 
 void CDasherInterfaceBase::SetStringParameter(int iParameter, const std::string & sValue) {
-  PreSetNotify(iParameter);
-  
+  PreSetNotify(iParameter, sValue);
   m_pSettingsStore->SetStringParameter(iParameter, sValue);
 };
 
@@ -815,7 +809,7 @@ void CDasherInterfaceBase::ResetParameter(int iParameter) {
 
 // We need to be able to get at the UserLog object from outside the interface
 CUserLog* CDasherInterfaceBase::GetUserLogPtr() {
-	return m_pUserLog;
+  return m_pUserLog;
 }
 
 void CDasherInterfaceBase::KeyDown(int iTime, int iId) {
@@ -833,14 +827,23 @@ void CDasherInterfaceBase::KeyUp(int iTime, int iId) {
 void CDasherInterfaceBase::CreateInputFilter()
 {
   if(m_pInputFilter) {
+    m_pInputFilter->Deactivate();
     m_pInputFilter->Unref();
     m_pInputFilter = NULL;
   }
 
   m_pInputFilter = (CInputFilter *)GetModuleByName(GetStringParameter(SP_INPUT_FILTER));
 
-  if(m_pInputFilter)
+  if(m_pInputFilter) {
     m_pInputFilter->Ref();
+    m_pInputFilter->Activate();
+  }
+  else {
+    // Fall back to a sensible alternative if for some reason the
+    // current choice isn't valid.
+    if(GetStringParameter(SP_INPUT_FILTER) != "Normal Control")
+      SetStringParameter(SP_INPUT_FILTER, "Normal Control");
+  }
 }
 
 void CDasherInterfaceBase::RegisterFactory(CModuleFactory *pFactory) {
@@ -862,7 +865,7 @@ void CDasherInterfaceBase::CreateFactories() {
   RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CEyetrackerFilter(m_pEventHandler, m_pSettingsStore, this, m_pDasherModel)));
   RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CClickFilter(m_pEventHandler, m_pSettingsStore, this)));
   RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDynamicFilter(m_pEventHandler, m_pSettingsStore, this)));
-  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CTwoButtonDynamicFilter(m_pEventHandler, m_pSettingsStore, this)));
+  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CTwoButtonDynamicFilter(m_pEventHandler, m_pSettingsStore, this, 14, 1, "Two Button Dynamic Mode")));
   // TODO: specialist factory for button mode
   RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherButtons(m_pEventHandler, m_pSettingsStore, this, 5, 1, true,8, "Menu Mode")));
   RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherButtons(m_pEventHandler, m_pSettingsStore, this, 3, 0, false,10, "Direct Mode")));
@@ -894,3 +897,7 @@ void CDasherInterfaceBase::GetPermittedValues(int iParameter, std::vector<std::s
 void CDasherInterfaceBase::StartShutdown() {
   m_bShutdownLock = true;
 }
+
+bool CDasherInterfaceBase::GetModuleSettings(const std::string &strName, SModuleSettings **pSettings, int *iCount) {
+  return GetModuleByName(strName)->GetSettings(pSettings, iCount);
+}
