@@ -13,7 +13,7 @@ class CDefaultFilter : public CInputFilter {
   virtual void HandleEvent(Dasher::CEvent * pEvent);
 
   virtual bool DecorateView(CDasherView *pView);
-  virtual void Timer(int Time, CDasherView *m_pDasherView, CDasherModel *m_pDasherModel);
+  virtual bool Timer(int Time, CDasherView *m_pDasherView, CDasherModel *m_pDasherModel);
   virtual void KeyDown(int iTime, int iId, CDasherModel *pModel);
 
  protected:
