@@ -6,14 +6,13 @@
 //
 /////////////////////////////////////////////////////////////////////////////
 
-#include "WinCommon.h"
+#include "..\Common\WinCommon.h"
 
 #include "Canvas.h"
-#include "Edit.h"
+//#include "Edit.h"
 #include "../Dasher.h"
 #include "../DasherInterface.h"
-#include "../DasherMouseInput.h"
-#include "../Sockets/SocketInput.h"
+
 
 
 #define PRESSED		0x8000
@@ -25,10 +24,10 @@ using namespace Dasher;
 
 /////////////////////////////////////////////////////////////////////////////
 
-CCanvas::CCanvas(CDasherInterface *DI)
+CCanvas::CCanvas(CDasherInterface *DI, Dasher::CEventHandler *pEventHandler, CSettingsStore *pSettingsStore)
 	:m_pDasherInterface(DI), imousex(0), imousey(0), buttonnum(0), mousepostime(0) ,
 	m_dwTicksLastEvent(0), m_bButtonDown(false), m_pScreen(0),
-	CDasherComponent(DI->GetEventHandler(), DI->CreateSettingsStore())
+	CDasherComponent(pEventHandler, pSettingsStore)
 
 {
 
@@ -56,20 +55,32 @@ HWND CCanvas::Create(HWND hParent)
 	// NB We create the SocketInput object now, even if socket input is not enabled, because
 	// we can't safely create it later in response to a parameter change event (because it itself
 	// needs to register an event listener when it constructs itself).
-	m_pSocketInput = new CSocketInput(m_pDasherInterface->GetEventHandler(), m_pDasherInterface->CreateSettingsStore()); // CreateSettingsStore only creates a new one if there isn't one there already
-	m_pMouseInput = new CDasherMouseInput(hWnd);
 
-	if(m_pDasherInterface->GetBoolParameter(BP_SOCKET_INPUT_ENABLE)) 
+  // CreateSettingsStore only creates a new one if there isn't one there already
+
+
+// TODO: Reimplement?
+
+ //	m_pSocketInput = (CSocketInput *)m_pDasherInterface->GetModule(1);
+ // m_pSocketInput->Ref();
+
+	//m_pMouseInput = (CDasherMouseInput *)m_pDasherInterface->GetModule(0); 
+ // m_pMouseInput->Ref();
+
+	/*if(m_pDasherInterface->GetBoolParameter(BP_SOCKET_INPUT_ENABLE)) 
 	{
 		m_pSocketInput->StartListening();
-		m_pDasherInterface->SetInput(m_pSocketInput);
+		m_pDasherInterface->SetInput(1);
 	}
 	else {
-		m_pDasherInterface->SetInput(m_pMouseInput);
-	}
+		m_pDasherInterface->SetInput(0);
+	}*/
 
 
+  m_pKeyboardHelper = new CKeyboardHelper;
+
 	m_pScreen = new CScreen(m_hdc, 300, 300);
+  m_pScreen->SetFont(m_pDasherInterface->GetStringParameter(SP_DASHER_FONT));
 	//ReleaseDC(m_hwnd,m_hDC);
 	m_pDasherInterface->ChangeScreen(m_pScreen);
 
@@ -135,12 +146,15 @@ LRESULT CCanvas::OnDestroy(UINT message, WPARAM wParam, LPARAM lParam, BOOL& bHa
 
 CCanvas::~CCanvas() {
   delete m_pScreen;
-  if(m_pMouseInput != NULL) {
-    delete m_pMouseInput;
-  }
-  if(m_pSocketInput != NULL) {
-    delete m_pSocketInput;
-  }
+  //if(m_pMouseInput != NULL) {
+  //  delete m_pMouseInput;
+  //}
+  //if(m_pSocketInput != NULL) {
+  //  delete m_pSocketInput;
+  //}
+
+  if(m_pKeyboardHelper)
+    delete m_pKeyboardHelper;
 }
 
 void CCanvas::Move(int x, int y, int Width, int Height) 
@@ -185,6 +199,17 @@ LRESULT CCanvas::OnPaint(UINT message, WPARAM wParam, LPARAM lParam, BOOL& bHand
 LRESULT CCanvas::OnKeyUp(UINT message, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
 {
 	bHandled = TRUE;
+
+  int iKeyVal(-1);
+
+  if(m_pKeyboardHelper)
+    iKeyVal = m_pKeyboardHelper->ConvertKeyCode(wParam);
+
+  if(iKeyVal != -1) {
+    m_pDasherInterface->KeyUp(GetTickCount(), iKeyVal);
+    return 0;
+  }
+
 	switch(wParam) 
 	{
 	case VK_SHIFT:
@@ -198,20 +223,8 @@ LRESULT CCanvas::OnKeyUp(UINT message, WPARAM wParam, LPARAM lParam, BOOL& bHand
 			m_pDasherInterface->SetLongParameter(LP_BOOSTFACTOR, 175);
 		else
 			m_pDasherInterface->SetLongParameter(LP_BOOSTFACTOR, 100);
+
 		return 0;
-      // Button mode keys
-  case 0x41:
-    m_pDasherInterface->KeyUp(GetTickCount(), 1);
-    return 0;
-  case 0x53:
-    m_pDasherInterface->KeyUp(GetTickCount(), 2);
-    return 0;
-  case 0x57:
-    m_pDasherInterface->KeyUp(GetTickCount(), 3);
-    return 0;
-  case 0x58:
-    m_pDasherInterface->KeyUp(GetTickCount(), 4);
-    return 0;
 	default:
 		return 0;		  
 	}
@@ -226,6 +239,16 @@ LRESULT CCanvas::OnKeyDown(UINT message, WPARAM wParam, LPARAM lParam, BOOL& bHa
 
   bHandled = TRUE;
 
+  int iKeyVal(-1);
+
+  if(m_pKeyboardHelper)
+    iKeyVal = m_pKeyboardHelper->ConvertKeyCode(wParam);
+
+  if(iKeyVal != -1) {
+    m_pDasherInterface->KeyDown(GetTickCount(), iKeyVal);
+    return 0;
+  }
+
 	switch (wParam) {
   // Space, for start/stop events etc.
 	case VK_SPACE:
@@ -406,6 +429,7 @@ LRESULT CCanvas::OnSize(UINT message, WPARAM wParam, LPARAM lParam, BOOL& bHandl
 	if (LOWORD(lParam)>0 && HIWORD(lParam) >0)
 	{
 		m_pScreen = new CScreen(m_hdc, LOWORD(lParam), HIWORD(lParam));
+    m_pScreen->SetFont(m_pDasherInterface->GetStringParameter(SP_DASHER_FONT));
 		m_pDasherInterface->ChangeScreen(m_pScreen);
 		InvalidateRect( NULL, FALSE);
 	}
@@ -613,28 +637,24 @@ void CCanvas::HandleEvent(Dasher::CEvent *pEvent) {
   if(pEvent->m_iEventType == 1) {
     Dasher::CParameterNotificationEvent * pEvt(static_cast < Dasher::CParameterNotificationEvent * >(pEvent));
     switch (pEvt->m_iParameter) {
-    case BP_SOCKET_INPUT_ENABLE:
+      case SP_DASHER_FONT:
+        m_pScreen->SetFont(m_pDasherInterface->GetStringParameter(SP_DASHER_FONT));
+        break;
+  /*  case BP_SOCKET_INPUT_ENABLE:
       OutputDebugString(TEXT("Processing BP_SOCKET_INPUT_ENABLE change\n"));
       if(GetBoolParameter(BP_SOCKET_INPUT_ENABLE)) {
-        if(m_pSocketInput == NULL) { // shouldn't occur
-          OutputDebugString(TEXT("Created new SocketInput instance\n"));
-	        m_pSocketInput = new CSocketInput(m_pDasherInterface->GetEventHandler(), m_pDasherInterface->CreateSettingsStore()); // CreateSettingsStore only creates a new one if there isn't one there already
-        }
         if(!m_pSocketInput->isListening()) {
 	        m_pSocketInput->StartListening();
         } 
-        m_pDasherInterface->SetInput(m_pSocketInput);
+        m_pDasherInterface->SetInput(1);
       } 
       else {
-        if(m_pMouseInput == NULL) { // shouldn't occur
-	        m_pMouseInput = new CDasherMouseInput(m_hWnd);
-        }
         if(m_pSocketInput != NULL) {
 	        m_pSocketInput->StopListening();
         }
-        m_pDasherInterface->SetInput(m_pMouseInput);
+        m_pDasherInterface->SetInput(0);
       }
-      break;
+      break;*/
     default:
       break;
     }
